@@ -1,9 +1,8 @@
 import json
 import re
 
-import requests
-
 import apsw
+
 from mangoapi.base_site import Site
 
 regexes = {
@@ -22,10 +21,11 @@ class Mangasee(Site):
     search_table = None
 
     def __init__(self, keyval_store=None):
+        super().__init__()
         self.keyval_store = keyval_store
 
     def get_title(self, title_id):
-        resp = requests.get(f"https://mangasee123.com/manga/{title_id}", timeout=3)
+        resp = self.session.get(f"https://mangasee123.com/manga/{title_id}", timeout=3)
         assert resp.status_code == 200
         html = resp.text
         name = regexes["title_name"].search(html).group(1).strip()
@@ -54,7 +54,7 @@ class Mangasee(Site):
         }
 
     def get_chapter(self, title_id, chapter_id):
-        resp = requests.get(
+        resp = self.session.get(
             f"https://mangasee123.com/read-online/{title_id}-chapter-{chapter_id}.html"
         )
         assert resp.status_code == 200
@@ -108,7 +108,7 @@ class Mangasee(Site):
                 )
             if not titles:
                 print("Fetching mangasee title list...", end="")
-                resp = requests.get("https://mangasee123.com/_search.php")
+                resp = self.session.get("https://mangasee123.com/_search.php")
                 print(" done")
                 titles = resp.json()
                 self.keyval_store.set("mangasee_titles", resp.text)
