@@ -9,6 +9,7 @@ import requests
 from flask import (
     Flask,
     flash,
+    jsonify,
     make_response,
     redirect,
     render_template,
@@ -18,18 +19,23 @@ from flask import (
 )
 
 from .conf import config
-from .decorators import require_login, toggle_has_read
+from .decorators import process_token, require_login
 from .persistence import (
+    create_token,
+    delete_token,
     follow,
     get_followed_titles,
     get_prev_next_chapters,
+    get_username,
     import_follows,
     load_chapter,
     load_title,
+    read,
     register_user,
     save_chapter,
     save_title,
     unfollow,
+    unread,
     verify_username_password,
 )
 from .source_sites import (
@@ -51,11 +57,18 @@ app.config.update(
 )
 
 
-@app.route("/")
-def home_view():
-    if session.get("user"):
-        return redirect(url_for("follows_view"))
-    return render_template("home.html")
+def _chapter_name(chapter: dict):
+    result = ""
+    if chapter.get("num_major") is not None:
+        result += "Ch. " if chapter.get("volume") else "Chapter "
+        result += str(chapter["num_major"])
+    if chapter.get("num_minor"):
+        result += '.{chapter["num_minor"]}'
+    if chapter.get("volume"):
+        result += f"Vol. {chapter['volume']}"
+    if chapter.get("name"):
+        result += f" - {chapter['name']}"
+    return result
 
 
 @app.route("/following", methods=["GET"])
@@ -67,21 +80,21 @@ def follows_view():
         if title["site"] == "mangadex":
             thumbnail = url_for("proxy_view", b64_url=_encode_proxy_url(thumbnail))
         title["thumbnail"] = thumbnail
-    return render_template("follows.html", titles=titles)
+    return render_template("old/follows.html", titles=titles)
 
 
 @app.route("/follow/<site>/<title_id>", methods=["POST"])
 @require_login
 def follow_view(site, title_id):
     follow(session["user"]["id"], site, title_id)
-    return redirect(url_for("title_view", site=site, title_id=title_id))
+    return redirect(url_for("spa_title_view", site=site, title_id=title_id))
 
 
 @app.route("/unfollow/<site>/<title_id>", methods=["POST"])
 @require_login
 def unfollow_view(site, title_id):
     unfollow(session["user"]["id"], site, title_id)
-    return redirect(url_for("title_view", site=site, title_id=title_id))
+    return redirect(url_for("spa_title_view", site=site, title_id=title_id))
 
 
 @app.route("/logout", methods=["POST"])
@@ -129,7 +142,7 @@ def auth_view():
                     status_code = 200
             return (
                 render_template(
-                    "auth.html",
+                    "old/auth.html",
                     register_username=username,
                     register_password=password,
                     register_confirm_password=confirm_password,
@@ -159,7 +172,7 @@ def auth_view():
 
             return (
                 render_template(
-                    "auth.html",
+                    "old/auth.html",
                     login_username=username,
                     login_password=password,
                     login_remember=remember,
@@ -170,61 +183,7 @@ def auth_view():
             )
 
     # Just a plain ol' GET request:
-    return render_template("auth.html")
-
-
-@app.route("/m/<site>/<title_id>")
-@toggle_has_read
-def title_view(site, title_id):
-    user = session.get("user", None)
-    user_id = user["id"] if user else None
-    title = load_title(site, title_id, user_id=user_id)
-    if not title:
-        print("Getting title", title_id)
-        title = get_title(site, title_id)
-        print("Saving title", title_id, "to db")
-        save_title(title)
-    else:
-        print("Loading title", title_id, "from db")
-    title["cover"] = title_cover(site, title_id, title["cover_ext"])
-    if site == "mangadex":
-        title["cover"] = url_for(
-            "proxy_view", b64_url=_encode_proxy_url(title["cover"])
-        )
-    title["source_url"] = title_source_url(site, title_id)
-    return render_template("title.html", **title)
-
-
-@app.route("/m/<site>/<title_id>/<chapter_id>")
-@toggle_has_read
-def chapter_view(site, title_id, chapter_id):
-    chapter = load_chapter(site, title_id, chapter_id)
-    if not chapter:
-        print("Getting chapter", chapter_id)
-        chapter = get_chapter(site, title_id, chapter_id)
-        save_chapter(chapter)
-    else:
-        print("Loading chapter", chapter_id, "from db")
-
-    if site in ("mangadex", "mangasee"):
-        chapter["pages"] = [
-            url_for("proxy_view", b64_url=_encode_proxy_url(p))
-            for p in chapter["pages"]
-        ]
-
-    # YIIIIKES
-    title = load_title(site, title_id)
-    title["cover"] = title_cover(site, title_id, title["cover_ext"])
-    if site == "mangadex":
-        title["cover"] = url_for(
-            "proxy_view", b64_url=_encode_proxy_url(title["cover"])
-        )
-    prev_chapter, next_chapter = get_prev_next_chapters(title, chapter)
-    chapter["prev_chapter"] = prev_chapter
-    chapter["next_chapter"] = next_chapter
-
-    chapter["site"] = site
-    return render_template("chapter.html", title=title, **chapter)
+    return render_template("old/auth.html")
 
 
 @app.route("/search")
@@ -239,7 +198,7 @@ def search_view():
             title["thumbnail"] = url_for(
                 "proxy_view", b64_url=_encode_proxy_url(title["thumbnail"])
             )
-    return render_template("search.html", results=results, query=query)
+    return render_template("old/search.html", results=results, query=query)
 
 
 @app.route("/proxy/<b64_url>")
@@ -304,7 +263,7 @@ def import_view():
 
             flash(f"Added {len(site_title_pairs)} follows.")
 
-    return render_template("import.html")
+    return render_template("old/import.html")
 
 
 def read_tachiyomi_follows(text: str) -> List[Tuple[str, str]]:
@@ -358,3 +317,242 @@ def ensure_titles(site_title_pairs: List[Tuple[str, str]]):
             title = future.result()
             save_title(title)
             print(f"Saved {title['site']}: {title['name']}")
+
+
+"""
+New Mithril-based SPA views follow
+"""
+
+
+@app.route("/")
+@app.route("/h")
+@app.route("/a")
+@app.route("/f")
+@app.route("/s")
+@app.route("/s/<query>")
+def home_view(query=None):
+    return render_template("spa.html")
+
+
+def _title(site, title_id, user_id=None):
+    title = load_title(site, title_id, user_id=user_id)
+    if not title:
+        print("Getting title", title_id)
+        title = get_title(site, title_id)
+        print("Saving title", title_id, "to db")
+        save_title(title)
+    else:
+        print("Loading title", title_id, "from db")
+    title["cover"] = title_cover(site, title_id, title["cover_ext"])
+    if site == "mangadex":
+        title["cover"] = url_for(
+            "proxy_view", b64_url=_encode_proxy_url(title["cover"])
+        )
+    title["source_url"] = title_source_url(site, title_id)
+    return title
+
+
+@app.route("/m/<site>/<title_id>")
+def spa_title_view(site, title_id):
+    title = _title(site, title_id)
+    return render_template(
+        "spa.html",
+        open_graph={
+            "title": title["name"],
+            "image": title["cover"],
+            "description": "\n".join(title["descriptions"]),
+        },
+    )
+
+
+@app.route("/m/<site>/<title_id>/<chapter_id>")
+def spa_chapter_view(site, title_id, chapter_id):
+    chapter = load_chapter(site, title_id, chapter_id)
+    if not chapter:
+        print("Getting chapter", chapter_id)
+        chapter = get_chapter(site, title_id, chapter_id)
+        save_chapter(chapter)
+    else:
+        print("Loading chapter", chapter_id, "from db")
+
+    # YIIIIKES
+    title = load_title(site, title_id)
+    title["cover"] = title_cover(site, title_id, title["cover_ext"])
+    if site == "mangadex":
+        title["cover"] = url_for(
+            "proxy_view", b64_url=_encode_proxy_url(title["cover"])
+        )
+
+    chapter["site"] = site
+    return render_template(
+        "spa.html",
+        open_graph={
+            "title": f'{_chapter_name(chapter)} - {title["name"]}',
+            "image": title["cover"],
+            "description": "\n".join(title["descriptions"]),
+        },
+    )
+
+
+@app.route("/api/title/<site>/<title_id>", methods=["GET"])
+@process_token(required=False)
+def api_title(site, title_id):
+    title = _title(site, title_id, user_id=request.user_id)
+    return title
+
+
+@app.route("/api/chapter/<site>/<title_id>/<chapter_id>", methods=["GET"])
+@process_token(required=False)
+def api_chapter(site, title_id, chapter_id):
+    chapter = load_chapter(site, title_id, chapter_id)
+    if not chapter:
+        print("Getting chapter", chapter_id)
+        chapter = get_chapter(site, title_id, chapter_id)
+        save_chapter(chapter)
+    else:
+        print("Loading chapter", chapter_id, "from db")
+
+    if site in ("mangadex", "mangasee"):
+        chapter["pages"] = [
+            url_for("proxy_view", b64_url=_encode_proxy_url(p))
+            for p in chapter["pages"]
+        ]
+
+    # YIIIIKES
+    title = load_title(site, title_id)
+    title["cover"] = title_cover(site, title_id, title["cover_ext"])
+    if site == "mangadex":
+        title["cover"] = url_for(
+            "proxy_view", b64_url=_encode_proxy_url(title["cover"])
+        )
+    prev_chapter, next_chapter = get_prev_next_chapters(title, chapter)
+    chapter["prev_chapter"] = prev_chapter
+    chapter["next_chapter"] = next_chapter
+    chapter["site"] = site
+    return chapter
+
+
+@app.route("/api/register", methods=["POST"])
+def api_register():
+    username = request.json["username"].strip()
+    password = request.json["password"]
+    message = None
+    if not (username and password):
+        message = "Empty field(s) spotted. Protip: spaces don't count."
+        status_code = 400
+    elif (
+        len(username) < 2
+        or len(username) > 15
+        or len(password) < 5
+        or len(password) > 50
+    ):
+        message = "Invalid username/password length. Username length should be 2~15, password 5~50."
+        status_code = 400
+    else:  # success!
+        err = register_user(username, password)
+        if err:
+            message = err
+            status_code = 400
+        else:
+            message = "Registration successful! You can login now."
+            status_code = 200
+    return (jsonify({"message": message}), status_code)
+
+
+@app.route("/api/login", methods=["POST"])
+def api_login():
+    username = request.json["username"].strip()
+    password = request.json["password"]
+    remember = request.json["remember"]
+
+    if not (username and password):
+        return (
+            jsonify({"message": "Empty field(s) spotted. Protip: spaces don't count."}),
+            400,
+        )
+
+    user_id = verify_username_password(username, password)
+    if not user_id:
+        return jsonify({"message": "Wrong username/password combination."}), 401
+
+    token = create_token(user_id, remember)
+    return jsonify({"user_id": user_id, "token": token}), 200
+
+
+@app.route("/api/verify-token", methods=["GET"])
+@process_token(required=True)
+def api_verify_token():
+    return {"user_id": request.user_id, "username": get_username(request.user_id)}, 200
+
+
+@app.route("/api/logout", methods=["POST"])
+@process_token(required=True)
+def api_logout():
+    num_deleted = delete_token(request.token)
+    if num_deleted != 1:
+        return jsonify({"message": "Invalid token."}), 401
+    return "{}", 200
+
+
+@app.route("/api/follows", methods=["GET"])
+@process_token(required=True)
+def api_follows():
+    titles = get_followed_titles(request.user_id)
+    for title in titles:
+        thumbnail = title_thumbnail(title["site"], title["id"])
+        if title["site"] == "mangadex":
+            thumbnail = url_for("proxy_view", b64_url=_encode_proxy_url(thumbnail))
+        title["thumbnail"] = thumbnail
+    return jsonify({"titles": titles})
+
+
+@app.route("/api/search/<query>", methods=["GET"])
+def api_search(query):
+    results = search_title_all_sites(query)
+
+    if "mangadex" in results:
+        for title in results["mangadex"]:
+            title["thumbnail"] = url_for(
+                "proxy_view", b64_url=_encode_proxy_url(title["thumbnail"])
+            )
+    return results
+
+
+@app.route("/api/follow", methods=["POST"])
+@process_token(required=True)
+def api_follow():
+    should_follow = request.json["follow"]
+    site = request.json["site"]
+    title_id = request.json["title_id"]
+
+    if should_follow:
+        follow(request.user_id, site, title_id)
+    else:
+        unfollow(request.user_id, site, title_id)
+
+    return jsonify({"follow": should_follow})
+
+
+@app.route("/api/read", methods=["POST"])
+@process_token(required=True)
+def api_read():
+    reads = request.json.get("read") or []
+    unreads = request.json.get("unread") or []
+    assert reads or unreads
+
+    if reads:
+        for r in reads:
+            read(
+                request.user_id, r["site"], r["title_id"], r["chapter_id"],
+            )
+    if unreads:
+        for u in unreads:
+            unread(
+                request.user_id, u["site"], u["title_id"], u["chapter_id"],
+            )
+    # TODO: rewrite read/unread to do bulk updates instead of n+1 queries like these.
+    # ... Or maybe not. SQLite doesn't mind.
+
+    # Also TODO: maybe a separate "read all from title" API would be cleaner & easier on
+    # FE side.
+    return {}
