@@ -1,4 +1,5 @@
 import json
+import secrets
 from typing import List, Tuple
 
 import apsw
@@ -79,10 +80,9 @@ def load_title(site, title_id, user_id=None):
                 """
                 SELECT r.chapter_id
                 FROM read r
-                  INNER JOIN chapter c ON c.id = r.chapter_id AND c.site = r.site
                 WHERE r.user_id = ?
-                  AND c.title_id = ?
-                  AND c.site = ?
+                  AND r.title_id = ?
+                  AND r.site = ?
                 ORDER BY r.updated_at;
                 """,
                 (user_id, title["id"], title["site"]),
@@ -247,27 +247,18 @@ def get_followed_titles(user_id):
 
         # n+1 queries cuz I don't give a f- actually I do, but sqlite's cool with it:
         # https://www.sqlite.org/np1queryprob.html
-        chapters_i_finished = run_sql_on_demand(
+        chapters_i_finished = run_sql(
             """
-            SELECT r.chapter_id
-            FROM read r
-                INNER JOIN chapter c ON c.id = r.chapter_id AND c.site = r.site
-            WHERE r.user_id = ?
-                AND c.title_id = ?
-                AND c.site = ?
-            ORDER BY c.num_major desc, c.num_minor desc;
+            SELECT chapter_id
+            FROM read
+            WHERE user_id = ?
+              AND title_id = ?
+              AND site = ?;
             """,
             (user_id, t["id"], t["site"]),
         )
-        # Cut off chapter list:
-        # only show chapters newer than the latest chapter that user has finished.
-        # Running a loop here instead of just picking the one latest finished chapter
-        # because source site may have deleted said chapter.
-        for finished_chapter_id in chapters_i_finished:
-            for i, ch in enumerate(chapters):
-                if finished_chapter_id == ch["id"]:
-                    chapters = chapters[:i]
-                    break
+        # Only show chapters that user hasn't read
+        chapters = [ch for ch in chapters if ch["id"] not in chapters_i_finished]
 
         t["chapters"] = chapters
 
@@ -335,3 +326,62 @@ def import_follows(user_id: int, site_title_pairs: List[Tuple[str, str]]):
         """,
         ((user_id, site, title_id) for site, title_id in site_title_pairs),
     )
+
+
+def create_token(user_id, remember=False):
+    lifespan = "+365 days" if remember else "+1 day"
+    token = secrets.token_urlsafe(64)
+    run_sql(
+        """
+        INSERT INTO token (user_id, token, lifespan) VALUES (?,?,?);
+        """,
+        (user_id, token, lifespan),
+    )
+    return token
+
+
+def verify_token(token):
+    """
+    Checks if there's a matching token that hasn't exceeded its lifespan.
+    If there's a match, refreshes its last_accessed_at value, effectively expanding
+    its life.
+    """
+    result = run_sql(
+        """
+        SELECT user_id FROM token
+        WHERE token=?
+          AND datetime(last_accessed_at, lifespan) > datetime('now');
+        """,
+        (token,),
+    )
+    user_id = result[0] if len(result) == 1 else None
+    if user_id:
+        run_sql(
+            "UPDATE token SET last_accessed_at = datetime('now') WHERE token=?;",
+            (token,),
+        )
+    return user_id
+
+
+def get_username(user_id):
+    result = run_sql("SELECT username FROM user WHERE id=?;", (user_id,))
+    assert len(result) == 1
+    return result[0]
+
+
+def delete_token(token):
+    num_deleted = run_sql(
+        "DELETE FROM token WHERE token=?;", (token,), return_num_affected=True,
+    )
+    return num_deleted
+
+
+def delete_expired_tokens():
+    num_deleted = run_sql(
+        """
+        DELETE FROM token
+        WHERE datetime(last_accessed_at, lifespan) < datetime('now');
+        """,
+        return_num_affected=True,
+    )
+    return num_deleted
