@@ -1,14 +1,15 @@
 import time
+from abc import ABC, abstractmethod
 from datetime import datetime, timedelta
 
-from .persistence import find_outdated_titles, save_title
+from .persistence import delete_expired_tokens, find_outdated_titles, save_title
 from .source_sites import get_title
 
 now = datetime.now
 
 
 def main_loop():
-    workers = [UpdateOutdatedTitles()]
+    workers = [UpdateOutdatedTitles(), DeleteExpiredTokens()]
 
     while True:
         for worker in workers:
@@ -19,7 +20,7 @@ def main_loop():
         time.sleep(5)
 
 
-class Worker:
+class Worker(ABC):
     interval = timedelta(days=1)
 
     def __init__(self):
@@ -28,8 +29,9 @@ class Worker:
     def should_run(self):
         return now() - self.last_run >= self.interval
 
+    @abstractmethod
     def run(self):
-        raise NotImplementedError()
+        pass
 
     def after_run(self):
         self.last_run = now()
@@ -44,3 +46,12 @@ class UpdateOutdatedTitles(Worker):
             updated_title = get_title(title["site"], title["id"])
             save_title(updated_title)
             print(" done")
+
+
+class DeleteExpiredTokens(Worker):
+    interval = timedelta(days=1)
+
+    def run(self):
+        num_deleted = delete_expired_tokens()
+        if num_deleted > 0:
+            print("Deleted", num_deleted, "tokens")
